/////////////////////////////////////////////////////////////////////////////
//
// wa_viz.h : Declaration of the CWa_viz
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __WA_VIZ_H_
#define __WA_VIZ_H_

#include "resource.h"
#include "effects.h"
#include "wa_viz_h.h"
#include "winamp_vis.h"
#include "wa_ipc.h"

// preset values
enum {
    PRESET_BARS = 0,
    PRESET_SCOPE,
    PRESET_COUNT
};

/////////////////////////////////////////////////////////////////////////////
// CWa_viz
class ATL_NO_VTABLE CWa_viz : 
    public CComObjectRootEx<CComSingleThreadModel>,
    public CComCoClass<CWa_viz, &CLSID_Wa_viz>,
    public IDispatchImpl<IWa_viz, &IID_IWa_viz, &LIBID_WA_VIZLib>,
    public IWMPEffects2
{
private:
    COLORREF    m_clrForeground;    // foreground color
    LONG        m_nPreset;          // current preset

    HRESULT WzToColor(const WCHAR *pwszColor, COLORREF *pcrColor);
    HRESULT ColorToWz( BSTR* pbstrColor, COLORREF crColor);
    DWORD SwapBytes(DWORD dwRet);
	winampVisHeader *GetVisHeader();
	BOOL LoadPresets();

public:
    CWa_viz();
    ~CWa_viz();
	void ResizeModule();
	void InitModule();
	HWND GetWindow() { return m_hwndParent; }

DECLARE_REGISTRY_RESOURCEID(IDR_WA_VIZ)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CWa_viz)
    COM_INTERFACE_ENTRY(IWa_viz)
    COM_INTERFACE_ENTRY(IDispatch)
    COM_INTERFACE_ENTRY(IWMPEffects)
    COM_INTERFACE_ENTRY(IWMPEffects2)
END_COM_MAP()

public:

    // CComCoClass Overrides
    HRESULT FinalConstruct();
    void FinalRelease();

    // IWa_viz
    STDMETHOD(get_foregroundColor)(/*[out, retval]*/ BSTR *pVal);
    STDMETHOD(put_foregroundColor)(/*[in]*/ BSTR newVal);

    // IWMPEffects
    STDMETHOD(Render)(TimedLevel *pLevels, HDC hdc, RECT *rc);
    STDMETHOD(MediaInfo)(LONG lChannelCount, LONG lSampleRate, BSTR bstrTitle);
    STDMETHOD(GetCapabilities)(DWORD * pdwCapabilities);
    STDMETHOD(GoFullscreen)(BOOL fFullScreen) { return E_NOTIMPL; };
    STDMETHOD(RenderFullScreen)(TimedLevel *pLevels) { return E_NOTIMPL; };
    STDMETHOD(DisplayPropertyPage)(HWND hwndOwner);
    STDMETHOD(GetTitle)(BSTR *bstrTitle);
    STDMETHOD(GetPresetTitle)(LONG nPreset, BSTR *bstrPresetTitle);
    STDMETHOD(GetPresetCount)(LONG *pnPresetCount);
    STDMETHOD(SetCurrentPreset)(LONG nPreset);
    STDMETHOD(GetCurrentPreset)(LONG *pnPreset);

    // IWMPEffects2
    STDMETHOD(SetCore)(IWMPCore * pCore);
    STDMETHOD(Create)(HWND hwndParent);
    STDMETHOD(Destroy)();
    STDMETHOD(NotifyNewMedia)(IWMPMedia *pMedia);
    STDMETHOD(OnWindowMessage)(UINT msg, WPARAM WParam, LPARAM LParam, LRESULT *plResultParam );
    STDMETHOD(RenderWindowed)(TimedLevel *pLevels, BOOL fRequiredRender );

private:
    void         ReleaseCore();

    HWND                        m_hwndParent;
	HWND						m_hwndVis;
    CComPtr<IWMPCore>           m_spCore;
	HINSTANCE					m_hDLL;
	winampVisHeader				*m_visHdr;
	winampVisModule				*m_visMod;
	LONG						m_presetCount;
	BOOL						m_modInitialized;
	BOOL						m_configMode;
	char						m_winClassName[256];
};

#endif //__WA_VIZ_H_
